//======================================================================
//
// Copyright (C) 1996-1998 Mark Russinovich and Bryce Cogswell  !!!!!!!
//                              -----------     --------------
//
// MODIFIED : WANG Nianhua ,2006/10/10
//
//======================================================================
#include "ntddk.h"
#include "stdarg.h"
#include "stdio.h"
#include "..\gui\commdef.h"
#include "HdHook.H"

//----------------------------------------------------------------------
//                         GLOBALS
//----------------------------------------------------------------------
// our user-inteface device object
PDEVICE_OBJECT          GUIDevice;
BOOLEAN                 GUIActive = FALSE;

BOOLEAN                 IsHooked  = FALSE;
PSRVTABLE               ServiceTable;

ZWDEVICEIOCONTROLFILE RealZwDeviceIoControlFile;

UCHAR __DiskSerial[DISK_SERIAL_BUFF_LENGTH]={0};
UCHAR __ChangeTo  [DISK_SERIAL_BUFF_LENGTH]={0};


PUCHAR IsSubString(PUCHAR String, PUCHAR SubString ,ULONG StringLength ,ULONG SubStringLength)
{
    ULONG i,j;
    for(i=0;i<StringLength - SubStringLength +1 ;i++){
        for(j=0;j<SubStringLength;j++){
            if(String[i+j]!=SubString[j])
                break;
        }
        if(j==SubStringLength)  //match a substring
            return String+i;
    }
    return NULL;
}

//----------------------------------------------------------------------
//
//  Our own routine for ZwDeviceIocontrolFile
//  We change the hard disk serial number value requested by user
//
//----------------------------------------------------------------------
NTSTATUS HookZwDeviceIoControlFile(
                                  IN HANDLE FileHandle,
                                  IN HANDLE Event OPTIONAL,
                                  IN PIO_APC_ROUTINE ApcRoutine OPTIONAL,
                                  IN PVOID ApcContext OPTIONAL,
                                  OUT PIO_STATUS_BLOCK IoStatusBlock,
                                  IN ULONG IoControlCode,
                                  IN PVOID InputBuffer OPTIONAL,
                                  IN ULONG InputBufferLength,
                                  OUT PVOID OutputBuffer OPTIONAL,
                                  IN ULONG OutputBufferLength
                                  )
{
    NTSTATUS rc;

    rc = RealZwDeviceIoControlFile (
        FileHandle,
        Event,
        ApcRoutine,
        ApcContext,
        IoStatusBlock,
        IoControlCode,
        InputBuffer,
        InputBufferLength,
        OutputBuffer,
        OutputBufferLength
    );

    if((0x7c088 ==IoControlCode) && OutputBufferLength >DISK_SERIAL_BUFF_LENGTH){
        PUCHAR Locate = IsSubString(OutputBuffer,__DiskSerial,OutputBufferLength,DISK_SERIAL_BUFF_LENGTH);
        if(Locate){
            UCHAR i;
            for(i=0;i<20;i++){
                Locate[i]= __ChangeTo[i];
            }
        }
    }
    return(rc);
}


//----------------------------------------------------------------------
//
// Replaces entries in the system service table with pointers to
// our own hook routines. We save off the real routine addresses.
//
//----------------------------------------------------------------------
VOID HookStart( void )
{
    if( !IsHooked ) {
        RealZwDeviceIoControlFile = SYSCALL( ZwDeviceIoControlFile );
        SYSCALL( ZwDeviceIoControlFile ) = (PVOID) HookZwDeviceIoControlFile;
        IsHooked = TRUE;
    }
}


VOID HookStop( )
{
    if( IsHooked ) {
        SYSCALL( ZwDeviceIoControlFile ) = (PVOID) RealZwDeviceIoControlFile;
        IsHooked = FALSE;
    }
}


//======================================================================
//         D E V I C E - D R I V E R  R O U T I N E S
//======================================================================
BOOLEAN  HDHookDeviceControl( IN PFILE_OBJECT FileObject, IN BOOLEAN Wait,
                              IN PVOID InputBuffer, IN ULONG InputBufferLength,
                              OUT PVOID OutputBuffer, IN ULONG OutputBufferLength,
                              IN ULONG IoControlCode, OUT PIO_STATUS_BLOCK IoStatus,
                              IN PDEVICE_OBJECT DeviceObject ) {
    BOOLEAN                 retval = FALSE;
    ULONG i;

    // Its a message from our GUI!
    IoStatus->Status      = STATUS_SUCCESS; // Assume success
    IoStatus->Information = 0;              // Assume nothing returned


    switch ( IoControlCode ) {
    case HDHOOK_HOOK:
        HookStart();
        break;

    case HDHOOK_UNHOOK:
        HookStop();
        break;

    case HDHOOK_SETSELFVALUE:
      if( InputBufferLength < DISK_SERIAL_BUFF_LENGTH || InputBuffer == NULL){
        IoStatus->Status = STATUS_INVALID_PARAMETER;
        break;
      }
      for(i=0; i< DISK_SERIAL_BUFF_LENGTH ;i++)
        __DiskSerial[i] = ((UCHAR *)InputBuffer)[i];

        break;
    case HDHOOK_SETEMULABLEVALUE:
      if( InputBufferLength < DISK_SERIAL_BUFF_LENGTH || InputBuffer == NULL){
        IoStatus->Status = STATUS_INVALID_PARAMETER;
        break;
      }

      for(i=0;i< DISK_SERIAL_BUFF_LENGTH ;i++)
        __ChangeTo[i] = ((UCHAR *)InputBuffer)[i];

        break;
  case HDHOOK_VERSION:
        if ( OutputBufferLength < sizeof(ULONG) ||
             OutputBuffer == NULL ) {
            IoStatus->Status = STATUS_INVALID_PARAMETER;
            break;
        }

        *(ULONG *)OutputBuffer = REGMONVERSION;
        IoStatus->Information = sizeof(ULONG);
        break;

    default:
        IoStatus->Status = STATUS_INVALID_DEVICE_REQUEST;
        break;
    }
    return TRUE;
}


//----------------------------------------------------------------------
//
// In this routine we handle requests to our own device. The only
// requests we care about handling explicitely are IOCTL commands that
// we will get from the GUI. We also expect to get Create and Close
// commands when the GUI opens and closes communications with us.
//----------------------------------------------------------------------
NTSTATUS HDHookDispatch( IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp )
{
    PIO_STACK_LOCATION      irpStack;
    PVOID                   inputBuffer;
    PVOID                   outputBuffer;
    ULONG                   inputBufferLength;
    ULONG                   outputBufferLength;
    ULONG                   ioControlCode;
    WORK_QUEUE_ITEM         workItem;

    // Go ahead and set the request up as successful
    Irp->IoStatus.Status      = STATUS_SUCCESS;
    Irp->IoStatus.Information = 0;

    // Get a pointer to the current location in the Irp. This is where
    //     the function codes and parameters are located.
    irpStack = IoGetCurrentIrpStackLocation (Irp);

    // Get the pointer to the input/output buffer and its length
    inputBuffer             = Irp->AssociatedIrp.SystemBuffer;
    inputBufferLength       = irpStack->Parameters.DeviceIoControl.InputBufferLength;
    outputBuffer            = Irp->AssociatedIrp.SystemBuffer;
    outputBufferLength      = irpStack->Parameters.DeviceIoControl.OutputBufferLength;
    ioControlCode           = irpStack->Parameters.DeviceIoControl.IoControlCode;

    switch (irpStack->MajorFunction) {
    case IRP_MJ_CREATE:
        GUIActive = TRUE;
        break;

    case IRP_MJ_SHUTDOWN:
        break;

    case IRP_MJ_CLOSE:
        GUIActive = FALSE;
        break;

    case IRP_MJ_DEVICE_CONTROL:
        // See if the output buffer is really a user buffer that we
        // can just dump data into.
        if( IOCTL_TRANSFER_TYPE(ioControlCode) == METHOD_NEITHER ) {

            outputBuffer = Irp->UserBuffer;
        }

        // Its a request from the GUI
        HDHookDeviceControl( irpStack->FileObject, TRUE,
                             inputBuffer, inputBufferLength,
                             outputBuffer, outputBufferLength,
                             ioControlCode, &Irp->IoStatus, DeviceObject );
        break;
    }
    IoCompleteRequest( Irp, IO_NO_INCREMENT );
    return STATUS_SUCCESS;
}


//----------------------------------------------------------------------
//
// RegmonUnload
//
// Our job is done - time to leave.
//
//----------------------------------------------------------------------
VOID RegmonUnload( IN PDRIVER_OBJECT DriverObject )
{
    WCHAR                   deviceLinkBuffer[]  = L"\\DosDevices\\"DRIVER_NAME;
    UNICODE_STRING          deviceLinkUnicodeString;

    // Unhook the registry
    if( IsHooked ) HookStop();

    // Delete the symbolic link for our device
    RtlInitUnicodeString( &deviceLinkUnicodeString, deviceLinkBuffer );
    IoDeleteSymbolicLink( &deviceLinkUnicodeString );

    // Delete the device object
    IoDeleteDevice( DriverObject->DeviceObject );

}

//----------------------------------------------------------------------
//
// DriverEntry
//
// Installable driver initialization. Here we just set ourselves up.
//
//----------------------------------------------------------------------
NTSTATUS DriverEntry(IN PDRIVER_OBJECT DriverObject, IN PUNICODE_STRING RegistryPath )
{
    NTSTATUS                ntStatus;
    WCHAR                   deviceNameBuffer[]  = L"\\Device\\"DRIVER_NAME;
    UNICODE_STRING          deviceNameUnicodeString;
    WCHAR                   deviceLinkBuffer[]  = L"\\DosDevices\\"DRIVER_NAME;
    UNICODE_STRING          deviceLinkUnicodeString;
    WCHAR                   startValueBuffer[] = L"Start";
    UNICODE_STRING          startValueUnicodeString;

    UNICODE_STRING          registryPath;
    HANDLE                  driverKey;
    ULONG                   startType, demandStart;
    RTL_QUERY_REGISTRY_TABLE paramTable[2];
    OBJECT_ATTRIBUTES       objectAttributes;
    int                     i;

    // Query our start type to see if we are supposed to monitor starting
    // at boot time
    registryPath.Buffer = ExAllocatePool( PagedPool,
                                          RegistryPath->Length + sizeof(UNICODE_NULL));

    if (!registryPath.Buffer) {
        return STATUS_INSUFFICIENT_RESOURCES;
    }

    registryPath.Length = RegistryPath->Length + sizeof(UNICODE_NULL);
    registryPath.MaximumLength = registryPath.Length;

    RtlZeroMemory( registryPath.Buffer, registryPath.Length );

    RtlMoveMemory( registryPath.Buffer,  RegistryPath->Buffer,
                   RegistryPath->Length  );

    RtlZeroMemory( &paramTable[0], sizeof(paramTable));
    paramTable[0].Flags = RTL_QUERY_REGISTRY_DIRECT;
    paramTable[0].Name = L"Start";
    paramTable[0].EntryContext = &startType;
    paramTable[0].DefaultType = REG_DWORD;
    paramTable[0].DefaultData = &startType;
    paramTable[0].DefaultLength = sizeof(ULONG);

    RtlQueryRegistryValues( RTL_REGISTRY_ABSOLUTE,
                            registryPath.Buffer, &paramTable[0],
                            NULL, NULL  );

    // Set start type to demand start so that boot logging
    // only happens this boot (unless the user reconfigures it in
    // the GUI)
    InitializeObjectAttributes( &objectAttributes, RegistryPath,
                                OBJ_CASE_INSENSITIVE, NULL, NULL );
    ntStatus = ZwOpenKey( &driverKey, KEY_WRITE, &objectAttributes );
    if( NT_SUCCESS( ntStatus )) {

        demandStart = SERVICE_DEMAND_START;
        RtlInitUnicodeString( &startValueUnicodeString, startValueBuffer );
        ZwSetValueKey( driverKey, &startValueUnicodeString, 0, REG_DWORD,
                       &demandStart, sizeof(demandStart ));
        ZwClose( driverKey );
    }

    // Setup our name and symbolic link
    RtlInitUnicodeString (&deviceNameUnicodeString,
                          deviceNameBuffer );
    RtlInitUnicodeString (&deviceLinkUnicodeString,
                          deviceLinkBuffer );

    // Set up the device used for GUI communications
    ntStatus = IoCreateDevice ( DriverObject,
                                0,
                                &deviceNameUnicodeString,
                                FILE_DEVICE_REGMON,
                                0,
                                TRUE,
                                &GUIDevice );
    if (NT_SUCCESS(ntStatus)) {

        // Create a symbolic link that the GUI can specify to gain access
        // to this driver/device
        ntStatus = IoCreateSymbolicLink (&deviceLinkUnicodeString,
                                         &deviceNameUnicodeString );

        // Create dispatch points for all routines that must be handled
        DriverObject->MajorFunction[IRP_MJ_SHUTDOWN]        =
        DriverObject->MajorFunction[IRP_MJ_CREATE]          =
        DriverObject->MajorFunction[IRP_MJ_CLOSE]           =
        DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL]  = HDHookDispatch;
#if DBG
        DriverObject->DriverUnload                          = RegmonUnload;
#endif
    }
    if (!NT_SUCCESS(ntStatus)) {

        // Something went wrong, so clean up (free resources etc)
        if( GUIDevice ) IoDeleteDevice( GUIDevice );
        IoDeleteSymbolicLink( &deviceLinkUnicodeString );
        return ntStatus;
    }
    // Pointer to system table data structure is an NTOSKRNL export
    ServiceTable = KeServiceDescriptorTable;

    return STATUS_SUCCESS;
}

