#include <windows.h>
#include <shellapi.h>
#include <winioctl.h>
#include <stdio.h>
#include "GUI.h"
#include "commdef.h"

#define WM_NOTIFYICON  (WM_APP+100)
#define MAX_LOADSTRING 100

// Global Variables:
BOOL      __bIsHooked=FALSE;
HANDLE    __SysHandle;
HINSTANCE __hInstance;							// current instance
TCHAR szTitle[MAX_LOADSTRING];					// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];			// The title bar text

// Foward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	SettingProc(HWND, UINT, WPARAM, LPARAM);

BOOL                IconMessage(HWND,DWORD,PSTR);
BOOL                LoadDeviceDriver( const TCHAR * Name, const TCHAR * Path, HANDLE * lphDevice );
BOOL                UnloadDeviceDriver( const TCHAR * Name );
VOID                ReadPhysicalDriveOnNT( BYTE * );
BOOL                WriteResourceToFile(UINT ,UINT ,const char * );


int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
 	// TODO: Place code here.
	MSG msg;
	HACCEL hAccelTable;

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_GUI, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	//only support NT/XP
	DWORD	NTVersion;
	NTVersion = GetVersion();
	if( NTVersion >= 0x80000000 ){
		MessageBox(NULL,"Only Support NT/XP","Error",MB_OK|MB_ICONERROR);
		return FALSE;
	};


	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_GUI);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return msg.wParam;
}


ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_GUI);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= (LPCSTR)IDC_MENU;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
   HWND hWnd;

   __hInstance = hInstance; // Store instance handle in our global variable

   hWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

   if (!hWnd)
   {
      return FALSE;
   }
   //ShowWindow(hWnd, nCmdShow);
   //UpdateWindow(hWnd);

   return TRUE;
}


/************************************************************************/
//
//**  des : buffer length must be larger than DISK_SERIAL_BUFF_LENGTH*2 +1
//
/************************************************************************/
void Byte2String(BYTE * src,char * des)
{
	int i ;
	for(i=0;i < DISK_SERIAL_BUFF_LENGTH ; i++){
		sprintf(des+(i<<1),"%02X",src[i]);
	}
	des[DISK_SERIAL_BUFF_LENGTH*2] =0;
}

inline int Ascii2Hex(char c)
{
	if(c >='0' && c <='9')
		return (c-'0');
	else if (c>='a' && c <='f')
		return (c -'a') + 10;
	else if(c>='A' && c <='F')
		return c-'A' + 10;
	else return 0xff;
}
BOOL String2Byte(const char * src,BYTE * des)
{
	int i,high,low;
	for(i =0 ; i <DISK_SERIAL_BUFF_LENGTH ;i++ ){
		high=Ascii2Hex(src[i<<1]);
		if(high==0xff)
			return FALSE;
		low=Ascii2Hex(src[(i<<1) +1]);
		if(low==0xff)
			return FALSE;
		des[i] = (high <<4) +low;		
	}
	return TRUE;
}

void HookOn(HWND hWnd)
{
	DWORD dwDummy;

	DeviceIoControl(__SysHandle,HDHOOK_HOOK,NULL,0,NULL,0,&dwDummy,NULL) ;
	__bIsHooked=TRUE;
	IconMessage(hWnd,NIM_MODIFY,"Hook Is ON"); 
}

void HookOff(HWND hWnd)
{
	DWORD dwDummy;
	DeviceIoControl(__SysHandle,HDHOOK_UNHOOK,NULL,0,NULL,0,&dwDummy,NULL) ;
	__bIsHooked=FALSE;
	IconMessage(hWnd,NIM_MODIFY,"Hook Is Off"); 
}


void SetEmulableValue(BYTE * szValue)
{
	DWORD dwDummy;
	DeviceIoControl(__SysHandle,HDHOOK_SETEMULABLEVALUE,szValue,DISK_SERIAL_BUFF_LENGTH, NULL, 0, &dwDummy, NULL) ;
}

VOID Load(HWND hWnd)
{
	DWORD dwDummy, versionNumber;	
	TCHAR szBuffer[MAX_PATH];
	
	BYTE  szEmulSerial[] ={
		0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
		0x5A, 0x20, 0x46, 0x35, 0x31, 0x4D, 0x35, 0x32, 0x54, 0x31,
	};

	GetSystemDirectory(szBuffer,sizeof szBuffer);
	strcat(szBuffer,"\\Drivers\\");
	strcat(szBuffer,DRIVER_FILE_NAME);
	WriteResourceToFile( IDR_HDHOOKSYS, 100, szBuffer );
	
	if ( !LoadDeviceDriver(SERVICE_NAME,szBuffer,&__SysHandle) ){
		MessageBox(hWnd,"Open Device Driver Error!","Error",MB_OK|MB_ICONERROR);
		ExitProcess(0);
	}
	if ( ! DeviceIoControl(__SysHandle, HDHOOK_VERSION,
		NULL, 0, &versionNumber, sizeof(DWORD), &dwDummy, NULL ) ||versionNumber != REGMONVERSION ){
		MessageBox( hWnd, "Driver has a Wrong Version for This Program","Error",MB_OK|MB_ICONERROR);
		ExitProcess(0);
	}
	//Hook Off ,and then get self hard Serial #
	HookOff(hWnd);
	ReadPhysicalDriveOnNT((BYTE *)szBuffer);
	char szTemp[50];
	Byte2String((BYTE *)szBuffer,szTemp);
	WriteProfileString(PROFILE_APP_NAME,PROFILE_KEY_NAME_CUR,szTemp);
	DeviceIoControl(__SysHandle,HDHOOK_SETSELFVALUE,szBuffer,DISK_SERIAL_BUFF_LENGTH, NULL, 0, &dwDummy, NULL) ;
	//
	GetProfileString(PROFILE_APP_NAME,PROFILE_KEY_NAME_NEW,"",szBuffer,sizeof szBuffer);

	if( (strlen(szBuffer) != 40 ) ||  !String2Byte(szBuffer,(BYTE *)szTemp) ){
		// set a default value
		DeviceIoControl(__SysHandle,HDHOOK_SETEMULABLEVALUE,szEmulSerial,DISK_SERIAL_BUFF_LENGTH, NULL, 0, &dwDummy, NULL) ;
	}else{
		DeviceIoControl(__SysHandle,HDHOOK_SETEMULABLEVALUE,szTemp,      DISK_SERIAL_BUFF_LENGTH, NULL, 0, &dwDummy, NULL) ;
	}
	HookOn(hWnd);
}


LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	TCHAR szHello[MAX_LOADSTRING];
	int wmId, wmEvent;	
	HDC hdc;
	PAINTSTRUCT ps;


	LoadString(__hInstance, IDS_HELLO, szHello, MAX_LOADSTRING);

	switch (message) 
	{
	case WM_COMMAND:
		wmId    = LOWORD(wParam); 
		wmEvent = HIWORD(wParam); 
		// Parse the menu selections:
		switch (wmId)
		{
		case IDM_ABOUT:
			DialogBox(__hInstance, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
			break;
		case IDM_EXIT:
			DestroyWindow(hWnd);
			break;
		case IDM_HOOKOFF:
			HookOff(hWnd);
			break;
		case IDM_HOOKON:
			HookOn(hWnd);
			break;
		case IDM_SETTING:
			DialogBox(__hInstance, (LPCTSTR)IDD_SETTING, hWnd, (DLGPROC)SettingProc);
			break;
			
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
		}
		break;
	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		// TODO: Add any drawing code here...
		RECT rt;
		GetClientRect(hWnd, &rt);
		DrawText(hdc, szHello, strlen(szHello), &rt, DT_CENTER);
		EndPaint(hWnd, &ps);
		break;
	case WM_DESTROY:
		IconMessage(hWnd,NIM_DELETE,NULL);
		HookOff(hWnd);
		UnloadDeviceDriver(SERVICE_NAME );
		PostQuitMessage(0);
		break;
	case WM_CREATE:	
		Load(hWnd);
		IconMessage(hWnd,NIM_ADD,"Hook Is Off"); 
		break;
			
	case WM_NOTIFYICON:
		switch(lParam)
		{
		case WM_LBUTTONDBLCLK://˫ͼ
			SetForegroundWindow(hWnd);
			DialogBox(__hInstance, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
			return FALSE;
		case WM_RBUTTONUP:
			{
				POINT point;
				HMENU hMenu, hSubMenu;

				// Get mouse position
				GetCursorPos(&point);
				// Popup context menu
				hMenu = LoadMenu(__hInstance, MAKEINTRESOURCE(IDC_MENU));
				hSubMenu = GetSubMenu(hMenu, 0);
				if(__bIsHooked){
					//CheckMenuItem(hMenu,IDM_HOOKON,TRUE);
					SetMenuDefaultItem(hSubMenu, IDM_HOOKON, FALSE);
				}
				else{
					//CheckMenuItem(hMenu,IDM_HOOKOFF,TRUE);
					SetMenuDefaultItem(hSubMenu, IDM_HOOKOFF, FALSE);
				}
				
				SetForegroundWindow(hWnd);
				TrackPopupMenu(hSubMenu,TPM_LEFTBUTTON|TPM_RIGHTBUTTON|TPM_LEFTALIGN,
				point.x, point.y, 0, hWnd, NULL);
				PostMessage(hWnd, WM_NULL, 0, 0);
				DestroyMenu(hMenu);
			}
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
		}
		break; //NO USE
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

#define Edit_LimitText(hwndCtl, cchMax)    \
			((void)SNDMSG((hwndCtl), EM_LIMITTEXT, (WPARAM)(cchMax), 0L))
// Message handler for Setting box.
LRESULT CALLBACK SettingProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	char szBuffer[50];
	BYTE szTemp[50];
	int wmId, wmEvent;	

	switch (message)
	{
		case WM_INITDIALOG:
			GetProfileString(PROFILE_APP_NAME,PROFILE_KEY_NAME_CUR,"",szBuffer,sizeof szBuffer);
			SetDlgItemText(hDlg,IDC_CURDISKVALUE,szBuffer);
			Edit_LimitText(GetDlgItem(hDlg,IDC_NEWDISKVALUE),40);

			GetProfileString(PROFILE_APP_NAME,PROFILE_KEY_NAME_NEW,"",szBuffer,sizeof szBuffer);
			SetDlgItemText(hDlg,IDC_NEWDISKVALUE,szBuffer);
			
			return TRUE;

		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			if (wmId == IDOK || wmId == IDCANCEL) {
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			if(wmId== IDC_APPLY){
				//TO DO 
				GetDlgItemText(hDlg,IDC_NEWDISKVALUE,szBuffer,sizeof szBuffer);
				if(strlen(szBuffer) != 40){
					MessageBox(hDlg,"Your Setting Value Must Be 40 Characters ! " ,"Error",MB_OK|MB_ICONERROR);
					break;
				}
				if(!String2Byte(szBuffer,szTemp)){
					MessageBox(hDlg,"All Character Must Be 0~9 or a~f or A~F !" ,"Error",MB_OK|MB_ICONERROR);
					break;
				}
				SetEmulableValue(szTemp);
				WriteProfileString(PROFILE_APP_NAME,PROFILE_KEY_NAME_NEW,szBuffer);
				MessageBox(hDlg,"New Value has Set Successfully" ,"Success" ,MB_OK|MB_ICONINFORMATION);
				EndDialog(hDlg, LOWORD(wParam));
				return true;
			}
			break;
	}
    return FALSE;

}
// Mesage handler for about box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
				return TRUE;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
			{
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}

BOOL IconMessage(HWND hDlg,DWORD dwMessage,PSTR pszTip)
{	
    BOOL retval;
    NOTIFYICONDATA tnd;
    HICON hIcon;

	if(__bIsHooked)
		hIcon=LoadIcon(__hInstance,MAKEINTRESOURCE(IDI_HOOKON));
	else
		hIcon=LoadIcon(__hInstance,MAKEINTRESOURCE(IDI_HOOKOFF));
	
    tnd.cbSize=sizeof(NOTIFYICONDATA);
    tnd.hWnd=hDlg;
    tnd.uID= 0x123456; 
    tnd.uFlags=NIF_MESSAGE|NIF_ICON|NIF_TIP;
    tnd.hIcon=hIcon;
    tnd.uCallbackMessage=WM_NOTIFYICON;
    if(pszTip) {
        lstrcpyn(tnd.szTip,pszTip,sizeof(tnd.szTip));
    }
    else {
        tnd.szTip[0]='\0';
    }
    retval=Shell_NotifyIcon(dwMessage,&tnd);
    if(hIcon)
        DestroyIcon(hIcon);
    return retval;
}


BOOL WriteResourceToFile(UINT idResource,UINT idType,const char * filename)
{
	HRSRC hResInfo = FindResource(__hInstance, MAKEINTRESOURCE(idResource),MAKEINTRESOURCE(idType));
	if(!hResInfo)
		return false;

    HGLOBAL hgRes = LoadResource(__hInstance, hResInfo);
	if(!hgRes)
		return false;

    void *pvRes = LockResource(hgRes);
	if(!pvRes)
		return false;

    DWORD cbRes = SizeofResource(__hInstance, hResInfo);

    HANDLE hFile = CreateFile(filename, GENERIC_WRITE, 0, 0, CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL, 0);
    if( hFile == INVALID_HANDLE_VALUE )
	{
		int a=GetLastError();
    	return false;
	}
    	
    DWORD cbWritten;
    WriteFile(hFile, pvRes, cbRes, &cbWritten, 0);
    CloseHandle(hFile);
	return true;
}
